#!/usr/bin/env node
"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
var analyze_1 = require("../commands/analyze");
var serve_1 = require("../commands/serve");
var build_1 = require("../commands/build");
var setup_1 = require("../commands/setup");
var page_1 = require("../commands/page");
var path_1 = require("path");
var neodoc_1 = require("neodoc");
var cli = "\n  \uD83D\uDC19  Elmo\u00A0\u2013\u00A0Dynamic websites on static infrastructure.\n\n  Usage:\n\n    elmo [options]\n    elmo [options] setup [<dir>]\n    elmo [options] page <page>\n    elmo [options] build [<out>]\n    elmo [options] serve [<port>]\n    elmo [options] analyze\n\n  Commands:\n\n    setup:     scaffold a new project\n    page:      scaffold a new page\n    build:     build the project\n    serve:     start the development server\n    analyze:   print details about the build\n\n  Global Options:\n\n    --typescript   Setup a typescript project\n    --dir <dir>    Working directory\n    --dev          Development build\n    --h, --help    Print this screen\n    --version      Get the version\n";
main()["catch"](function (err) {
    console.error(err.stack || err.message);
    process.exit(1);
});
function main() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var args, rel;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    args = neodoc_1["default"].run(cli, { smartOptions: true, laxPlacement: true });
                    if (args.setup) {
                        return [2 /*return*/, setup_1["default"]({
                                root: args['<dir>'] || args['--dir'],
                                typescript: !!args['--typescript']
                            })];
                    }
                    if (!args.page) return [3 /*break*/, 2];
                    return [4 /*yield*/, page_1["default"]({
                            root: args['--dir'],
                            pageName: args['<page>'],
                            typescript: !!args['--typescript']
                        })];
                case 1:
                    _a.sent();
                    console.error('🐙  wrote: ./%s', args['<page>']);
                    return [2 /*return*/];
                case 2:
                    if (args.build) {
                        return [2 /*return*/, build_1["default"]({
                                root: args['--dir'],
                                environment: args['--dev'],
                                outpath: args['--out']
                            })];
                    }
                    if (!args.serve) return [3 /*break*/, 4];
                    return [4 /*yield*/, serve_1["default"]({
                            root: args['--dir'],
                            environment: args['--dev'],
                            port: parseInt(args['<port>'], 10)
                        })];
                case 3:
                    _a.sent();
                    rel = path_1.relative(process.cwd(), args['--dir']);
                    if (rel) {
                        console.error("\uD83D\uDC19  All set! Run `cd " + rel + " && elmo serve` to get started.");
                    }
                    else {
                        console.error("\uD83D\uDC19  All set! Run `elmo serve` to get started.");
                    }
                    return [2 /*return*/];
                case 4:
                    if (!args.analyze) return [3 /*break*/, 6];
                    return [4 /*yield*/, analyze_1["default"]({
                            root: args['--dir'],
                            environment: args['--dev']
                        })];
                case 5: return [2 /*return*/, _a.sent()];
                case 6:
                    console.error(cli);
                    return [2 /*return*/];
            }
        });
    });
}
