"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
var webpack_1 = require("../server/config/webpack");
var Webpack = require("webpack");
var tempy_1 = require("tempy");
var archy = require("archy");
function Analyze(params) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _this = this;
        var env, root, tmpdir, s, b, stats, browserStats, browser;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    env = params.environment || 'production';
                    root = params.root || process.cwd();
                    tmpdir = tempy_1.directory();
                    s = webpack_1["default"]({
                        environment: env,
                        target: 'server',
                        action: 'analyze',
                        outpath: tmpdir,
                        routes: [],
                        root: root
                    });
                    b = webpack_1["default"]({
                        environment: env,
                        target: 'browser',
                        action: 'analyze',
                        outpath: tmpdir,
                        routes: [],
                        root: root
                    });
                    return [4 /*yield*/, Promise.all([s, b].map(function (config) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                            var c, webpack;
                            return tslib_1.__generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0: return [4 /*yield*/, config];
                                    case 1:
                                        c = _a.sent();
                                        webpack = Webpack(c);
                                        return [2 /*return*/, new Promise(function (resolve, reject) {
                                                webpack.run(function (err, stats) {
                                                    if (err)
                                                        return reject(err);
                                                    if (stats.hasErrors()) {
                                                        return reject(new Error(stats.toJson().errors.join('\n\n')));
                                                    }
                                                    return resolve(stats);
                                                });
                                            })];
                                }
                            });
                        }); }))];
                case 1:
                    stats = _a.sent();
                    browserStats = stats[1];
                    browser = browserStats.toJson({ chunkModules: true });
                    console.error(archy(formatTree(buildTree(browser.modules))));
                    return [2 /*return*/];
            }
        });
    });
}
exports["default"] = Analyze;
function formatTree(tree) {
    var line = {
        label: tree.size ? tree.name + ": " + formatSize(tree.size) : tree.name,
        nodes: []
    };
    if (!tree.children.length)
        return line;
    line.nodes = tree.children.map(formatTree);
    return line;
}
function buildTree(modules) {
    var root = {
        name: 'root',
        children: []
    };
    modules.forEach(function (module) {
        // remove this module if either:
        // - index is null
        // - issued by extract-text-plugin
        var extractInIdentifier = module.identifier.indexOf('extract-text-webpack-plugin') !== -1;
        var extractInIssuer = module.issuer &&
            module.issuer.indexOf('extract-text-webpack-plugin') !== -1;
        if (extractInIdentifier || extractInIssuer || module.index === null) {
            return;
        }
        var mod = {
            id: module.id,
            name: module.name,
            size: module.size,
            // TODO: show who imported it
            // reasons: module.reasons,
            children: []
        };
        var filename = module.name;
        if (module.name.slice(0, 2) === './') {
            filename = filename.slice(2);
        }
        walk(mod, filename, root);
    });
    return root;
}
function walk(module, filename, parent) {
    var idx = filename.indexOf('/');
    // leaf node
    if (!~filename.indexOf('/')) {
        module.name = filename;
        parent.children.push(module);
        return;
    }
    var folder = filename.slice(0, idx);
    if (folder === '~') {
        folder = 'node_modules';
    }
    var childFolder = getChild(parent.children, folder);
    if (!childFolder) {
        childFolder = {
            name: folder,
            children: []
        };
        parent.children.push(childFolder);
    }
    walk(module, filename.slice(idx + 1), childFolder);
}
function getChild(arr, name) {
    for (var i = 0; i < arr.length; i++) {
        if (arr[i].name === name) {
            return arr[i];
        }
    }
    return;
}
function formatSize(size, precision) {
    if (precision === void 0) { precision = 1; }
    var kb = {
        label: 'k',
        value: 1024
    };
    var mb = {
        label: 'M',
        value: 1024 * 1024
    };
    var denominator;
    if (size >= mb.value) {
        denominator = mb;
    }
    else {
        denominator = kb;
        if (size < kb.value * 0.92 && precision === 0) {
            precision = 1;
        }
    }
    return (size / denominator.value).toFixed(precision) + denominator.label;
}
