"use strict";
//
exports.__esModule = true;
var tslib_1 = require("tslib");
/*
 * Module dependencies
 */
var WebpackDevMiddleware = require("webpack-dev-middleware");
var WebpackHotMiddleware = require("webpack-hot-middleware");
var webpack_1 = require("../server/config/webpack");
var routes_1 = require("../server/routes");
var Deferred = require("deferral");
var Webpack = require("webpack");
var express = require("express");
var path_1 = require("path");
var regexps = {};
var app = express();
function serve(params) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var outpath, port, routes, s, b, _a, server, browser, firstBuild, watcher, webpack, listening, http, err, closing;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    params.root = params.root || process.cwd();
                    outpath = path_1.resolve(params.root, 'build');
                    port = params.port || 3130;
                    return [4 /*yield*/, routes_1["default"](params.root)
                        // start loading the server-side config
                    ];
                case 1:
                    routes = _b.sent();
                    s = webpack_1["default"]({
                        environment: params.environment || 'development',
                        outpath: outpath,
                        root: params.root,
                        target: 'server',
                        action: 'serve',
                        routes: routes
                    });
                    b = webpack_1["default"]({
                        environment: params.environment || 'development',
                        outpath: outpath,
                        root: params.root,
                        target: 'browser',
                        action: 'serve',
                        routes: routes
                    });
                    return [4 /*yield*/, Promise.all([s, b])];
                case 2:
                    _a = _b.sent(), server = _a[0], browser = _a[1];
                    firstBuild = new Deferred();
                    watcher = Webpack(server).watch({
                        ignored: outpath
                    }, function (err, stats) {
                        if (err) {
                            console.error(err);
                            return firstBuild.reject(err);
                        }
                        firstBuild.resolve(stats);
                    });
                    // wait for the first build to complete
                    // and don't go any further if there's
                    // a build error
                    return [4 /*yield*/, firstBuild
                        // create the browser build
                    ];
                case 3:
                    // wait for the first build to complete
                    // and don't go any further if there's
                    // a build error
                    _b.sent();
                    webpack = Webpack(browser);
                    app.use(WebpackDevMiddleware(webpack, {
                        publicPath: '/',
                        logLevel: 'silent'
                    }));
                    app.use(WebpackHotMiddleware(webpack, {
                        log: false
                    }));
                    // support custom routing
                    if (routes.length) {
                        // rehydrate regexps
                        app.use(function (req, _res, next) {
                            if (req.method !== 'GET')
                                return next();
                            for (var i = 0, len = routes.length; i < len; i++) {
                                var route = routes[i];
                                var regexp = (regexps[route.regexp] =
                                    regexps[route.regexp] || new RegExp("" + route.regexp));
                                if (!regexp.test(req.path))
                                    continue;
                                req.url = route.path.replace(/\/+$/, '') + '/';
                                return next();
                            }
                            return next();
                        });
                    }
                    app.use(express.static(path_1.resolve(params.root, outpath || 'build')));
                    listening = new Deferred();
                    http = app.listen(port);
                    http.once('error', function (err) { return listening.resolve(err); });
                    return [4 /*yield*/, listening
                        // cleanup, not sure if we need to wait
                        // for it to finish or not
                    ];
                case 4:
                    err = _b.sent();
                    closing = new Deferred();
                    watcher.close(function () { return closing.resolve(); });
                    return [4 /*yield*/, closing
                        // throw the message again after cleanup
                    ];
                case 5:
                    _b.sent();
                    // throw the message again after cleanup
                    throw err;
            }
        });
    });
}
exports["default"] = serve;
