"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
var preact_1 = require("preact"); // eslint-disable-line
var Document = /** @class */ (function (_super) {
    tslib_1.__extends(Document, _super);
    function Document() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Document.prototype.getChildContext = function () {
        return { _documentProps: this.props };
    };
    Document.prototype.render = function () {
        return (preact_1.h("html", null,
            preact_1.h(Head, null,
                preact_1.h("meta", { name: "viewport", content: "width=device-width, initial-scale=1" }),
                preact_1.h("link", { rel: "shortcut icon", type: "image/x-icon", href: "/static/favicon.ico" }),
                preact_1.h("meta", { name: "fragment", content: "!" })),
            preact_1.h("body", null,
                preact_1.h(Page, null),
                preact_1.h(Scripts, null))));
    };
    return Document;
}(preact_1.Component));
exports["default"] = Document;
var Head = /** @class */ (function (_super) {
    tslib_1.__extends(Head, _super);
    function Head() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Head.prototype.render = function (_a) {
        var children = _a.children;
        if (!children)
            return null;
        // prepend children if there's <Head />'s in the tree
        if (!this.context._documentProps)
            return preact_1.h("head", null, children);
        var _b = this.context._documentProps, head = _b.head, style = _b.style;
        if (head) {
            children = (head || []).concat(children).filter(unique()).reverse();
        }
        if (style)
            children = (children || []).concat(style);
        return preact_1.h("head", null, children || []);
    };
    return Head;
}(preact_1.Component));
exports.Head = Head;
var Page = /** @class */ (function (_super) {
    tslib_1.__extends(Page, _super);
    function Page() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Page.prototype.render = function () {
        var page = this.context._documentProps.page;
        return preact_1.h("div", { id: "elmo.page", dangerouslySetInnerHTML: { __html: page } });
    };
    return Page;
}(preact_1.Component));
exports.Page = Page;
var Scripts = /** @class */ (function (_super) {
    tslib_1.__extends(Scripts, _super);
    function Scripts() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Scripts.prototype.render = function () {
        var filepath = this.context._documentProps.filepath;
        return (preact_1.h("div", { id: "elmo.scripts" },
            preact_1.h("script", { type: "text/javascript", dangerouslySetInnerHTML: { __html: 'window.prerenderReady=false;' } }),
            preact_1.h("script", { type: "text/javascript", src: "/common.js" }),
            preact_1.h("script", { type: "text/javascript", src: "/" + filepath })));
    };
    return Scripts;
}(preact_1.Component));
exports.Scripts = Scripts;
var METATYPES = ['name', 'httpEquiv', 'charSet', 'itemProp'];
// returns a function for filtering head child elements
// which shouldn't be duplicated, like <title/>.
// TODO: typescript
function unique() {
    var tags = [];
    var metaTypes = [];
    var metaCategories = {};
    return function (h) {
        switch (h.nodeName) {
            case 'title':
            case 'base':
                if (~tags.indexOf(h.nodeName))
                    return false;
                tags.push(h.nodeName);
                break;
            case 'meta':
                for (var i = 0, len = METATYPES.length; i < len; i++) {
                    var metatype = METATYPES[i];
                    if (!h.attributes.hasOwnProperty(metatype))
                        continue;
                    if (metatype === 'charSet') {
                        if (~metaTypes.indexOf(metatype))
                            return false;
                        metaTypes.push(metatype);
                    }
                    else {
                        var category = h.attributes[metatype];
                        var categories = metaCategories[metatype] || [];
                        if (~categories.indexOf(category))
                            return false;
                        categories.push(category);
                        metaCategories[metatype] = categories;
                    }
                }
                break;
        }
        return true;
    };
}
// Add in the static references
Document.Head = Head;
Document.Page = Page;
Document.Scripts = Scripts;
