"use strict";
//
exports.__esModule = true;
var tslib_1 = require("tslib");
var static_1 = require("../plugins/static");
var FriendlyErrors = require("friendly-errors-webpack-plugin");
var CopyWebpackPlugin = require("copy-webpack-plugin");
var readPkg = require("read-pkg-up");
// import { CheckerPlugin } from 'awesome-typescript-loader'
var path_1 = require("path");
var fs_1 = require("mz/fs");
var route_1 = require("../../lib/route");
var webpack = require("webpack");
var globby = require("globby");
var routes_1 = require("../routes");
/**
 * Environment Variables
 */
function Environment() {
    return Object.keys(process.env).reduce(function (o, k) {
        o['process.env.' + k] = JSON.stringify(process.env[k]);
        return o;
    }, {});
}
// NOTE: these are required server-side
// so the server will require them directly
// and know where they are.
// (e.g. require('elmo/router'))
function ServerAliases(projectRoot, elmoRoot) {
    // resolve the project's preact first
    // then look for elmo's preact if
    // we can't find it
    var preact = resolves(path_1.join(projectRoot, 'node_modules', 'preact'), path_1.join(elmoRoot, 'node_modules', 'preact')) || 'preact';
    return {
        react: preact,
        'react-dom': preact,
        'preact': preact,
        'elmo': path_1.join(elmoRoot, 'dist', 'src', 'elmo.js'),
        'styled-tsx': path_1.join(elmoRoot, 'dist', 'src', 'styled-tsx', 'style.js')
    };
}
// NOTE: these are required client-side
// (e.g. import Router from 'elmo/router')
// these should only be requires that are
// added to the build. the public-facing
// aliases are here to allow us to narrow
// down our elmo typescript loader
function BrowserAliases(_projectRoot, elmoRoot) {
    return {
        unfetch: require.resolve('unfetch'),
        'promise-polyfill': require.resolve('promise-polyfill'),
        react: require.resolve('preact'),
        'react-dom': require.resolve('preact'),
        'elmo': path_1.join(elmoRoot, 'src', 'elmo.ts'),
        'styled-tsx': path_1.join(elmoRoot, 'src', 'styled-tsx', 'style.ts')
    };
}
/**
 * Webpack Config
 */
function Webpack(params) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var pkgPath, ELMO_ROOT, ELMO_JS_SRC, ELMO_TS_SRC, target, environment, action, routes, root, outpath, serverAliases, browserAliases, config, pages, publicPages, common, documentPage, rules, plugins, constants, _a, _b, _c, _d, _f, _g, _h, _j, hasAssets;
        return tslib_1.__generator(this, function (_k) {
            switch (_k.label) {
                case 0: return [4 /*yield*/, readPkg({ cwd: __dirname })];
                case 1:
                    pkgPath = (_k.sent()).path;
                    ELMO_ROOT = path_1.dirname(pkgPath);
                    ELMO_JS_SRC = path_1.join(ELMO_ROOT, 'dist', 'src');
                    ELMO_TS_SRC = path_1.join(ELMO_ROOT, 'src');
                    target = params.target, environment = params.environment, action = params.action, routes = params.routes;
                    root = path_1.resolve(process.cwd(), params.root || '.');
                    outpath = path_1.resolve(root, params.outpath || 'build');
                    serverAliases = ServerAliases(root, ELMO_ROOT);
                    browserAliases = BrowserAliases(root, ELMO_ROOT);
                    config = {
                        context: root,
                        module: { rules: [] },
                        plugins: [],
                        entry: {}
                    };
                    return [4 /*yield*/, globby(path_1.join(root, 'pages', '**', '*.{js,jsx,ts,tsx}'))];
                case 2:
                    pages = _k.sent();
                    publicPages = pages.filter(function (page) { return path_1.basename(page)[0] !== '_'; });
                    common = [];
                    // setup the entries
                    config.entry = publicPages.reduce(function (obj, page) {
                        // slice off first "/"
                        var route = route_1["default"](root, page).slice(1);
                        var name = path_1.join(route, 'index.js');
                        var bundle = [].concat(common);
                        // only include webpack hot middleware when we're
                        // not in production and using the dev server
                        // and we're building for the browser
                        if (action === 'serve' &&
                            environment !== 'production' &&
                            target === 'browser') {
                            bundle = bundle.concat('webpack-hot-middleware/client?path=/__webpack_hmr&noInfo=true&name=' +
                                name);
                        }
                        obj[name] = bundle.concat(page);
                        return obj;
                    }, {});
                    if (target !== 'browser') {
                        config.target = 'node';
                        // this is to ensure that we use node's actual require(...)
                        // which is needed for styled-jsx, but should probably be the
                        // default anyway.
                        config.externals = [
                            function (_parent, request, fn) {
                                if (serverAliases[request]) {
                                    return fn(null, serverAliases[request]);
                                }
                                else if (request[0] !== '.' && request[0] !== '/') {
                                    // resolve server-side node modules relative to 
                                    // the project's root before trying elmo's root.
                                    // this matters when we run npm link elmo
                                    var node_module = resolves(path_1.join(root, 'node_modules', request), path_1.join(ELMO_ROOT, 'node_modules', request));
                                    if (!node_module) {
                                        return fn(new Error('couldn\'t find: ' + request), undefined);
                                    }
                                    return fn(null, node_module);
                                }
                                else {
                                    // pass relative paths through (don't externalize)
                                    return fn(null, undefined);
                                }
                            }
                        ];
                    }
                    if (target === 'browser') {
                        config.node = {
                            console: false,
                            global: false,
                            process: false,
                            __filename: true,
                            __dirname: true,
                            Buffer: false,
                            setImmediate: false,
                            dns: false,
                            fs: false,
                            path: true,
                            url: false
                        };
                    }
                    documentPage = pages.find(function (p) { return path_1.basename(p, path_1.extname(p)) === '_document'; }) ||
                        path_1.join(ELMO_JS_SRC, 'lib', 'document.js');
                    // add the document as an entry if it exists
                    if (target === 'server') {
                        config.entry['document.js'] = common.concat(documentPage);
                    }
                    // resolves
                    config.resolve = {
                        extensions: ['.ts', '.tsx', '.js', '.jsx'],
                        modules: [path_1.join(root, 'node_modules'), path_1.join(ELMO_ROOT, 'node_modules')],
                        alias: browserAliases
                    };
                    // where to resolve loaders
                    config.resolveLoader = {
                        modules: [path_1.join(root, 'node_modules'), path_1.join(ELMO_ROOT, 'node_modules')]
                    };
                    /**
                     * Output
                     */
                    config.output = {
                        publicPath: '/',
                        filename: '[name]',
                        path: outpath,
                        libraryTarget: 'umd'
                    };
                    rules = config.module.rules;
                    // support json files
                    rules.push({
                        test: /\.json$/,
                        use: ['json-loader']
                    });
                    // compile our project's typescript files
                    rules.push({
                        test: /\.tsx?$/,
                        exclude: /(node_modules|bower_components)/,
                        loader: path_1.join(ELMO_JS_SRC, 'server', 'loaders', 'typescript.js'),
                        query: { environment: environment, target: target }
                    });
                    // certain typescript files are allowed to be compiled
                    // by the frontend configuration. allow certain
                    // modules to be compiled from elmo's ts source
                    rules.push({
                        test: /\.tsx?$/,
                        include: [
                            path_1.join(ELMO_TS_SRC, 'elmo.ts'),
                            path_1.join(ELMO_TS_SRC, 'lib'),
                            path_1.join(ELMO_TS_SRC, 'client'),
                            path_1.join(ELMO_TS_SRC, 'styled-tsx')
                        ],
                        loader: path_1.join(ELMO_JS_SRC, 'server', 'loaders', 'typescript.js'),
                        query: { environment: environment, target: target }
                    });
                    // always polyfill to find bugs faster
                    // right now just polyfilling fetch and promise
                    if (target === 'browser') {
                        rules.push({
                            test: /\.[tj]sx?$/,
                            include: publicPages.map(function (page) { return path_1.resolve(root, page); }),
                            use: [path_1.join(ELMO_JS_SRC, 'server', 'loaders', 'polyfills.js')]
                        });
                    }
                    // browser-side loaders
                    if (target === 'browser') {
                        // boot script for client-side, only added to the top-level pages
                        rules.push({
                            test: /\.[jt]sx?$/,
                            include: publicPages.map(function (page) { return path_1.resolve(root, page); }),
                            use: {
                                loader: path_1.join(ELMO_JS_SRC, 'server', 'loaders', 'boot.js'),
                                options: {
                                    context: root
                                }
                            }
                        });
                    }
                    plugins = config.plugins || [];
                    constants = Environment();
                    // attach the environment to the NODE_ENV
                    constants['process.env.NODE_ENV'] = JSON.stringify(constants['process.env.NODE_ENV'] || environment);
                    if (!(target === 'browser')) return [3 /*break*/, 6];
                    _b = (_a = Object).assign;
                    _c = [constants];
                    _d = {};
                    _f = 'window._ELMO_ROUTES_';
                    _h = (_g = JSON).stringify;
                    if (!routes.length) return [3 /*break*/, 3];
                    _j = routes;
                    return [3 /*break*/, 5];
                case 3: return [4 /*yield*/, routes_1["default"](root)];
                case 4:
                    _j = _k.sent();
                    _k.label = 5;
                case 5:
                    constants = _b.apply(_a, _c.concat([(_d[_f] = _h.apply(_g, [_j]),
                            _d)]));
                    _k.label = 6;
                case 6:
                    // Define environment variables
                    plugins.push(new webpack.DefinePlugin(constants));
                    // Needed this plugin to detect a `--watch` mode
                    // for typescript files
                    // if (environment === 'development' && action === 'serve') {
                    //   plugins.push(new CheckerPlugin())
                    // }
                    // Don't write files if there are errors
                    plugins.push(new webpack.NoEmitOnErrorsPlugin());
                    // use names instead of IDs
                    plugins.push(new webpack.NamedModulesPlugin());
                    // server-side
                    if (target === 'server') {
                        plugins.push(new static_1["default"]({
                            elmoRoot: ELMO_ROOT,
                            routes: routes,
                            environment: environment,
                            outpath: outpath,
                            target: target,
                            root: root,
                            action: action
                        }, {
                            'document.js': documentPage
                            // TODO add error & 404 page
                        }));
                    }
                    else if (target === 'browser') {
                        plugins.push(new webpack.optimize.CommonsChunkPlugin({
                            name: 'common',
                            filename: 'common.js'
                        }));
                    }
                    return [4 /*yield*/, fs_1.exists(path_1.join(root, 'static'))];
                case 7:
                    hasAssets = _k.sent();
                    if (target === 'browser' && hasAssets) {
                        plugins.push(new CopyWebpackPlugin([
                            {
                                context: root,
                                from: './static',
                                to: './static',
                                ignore: ['.*']
                            }
                        ]));
                    }
                    // when serving, show friendly errors
                    if (action === 'serve' && target === 'browser') {
                        plugins.push(new FriendlyErrors());
                    }
                    // enable hot module replacement when in dev server in the browser
                    if (action === 'serve' &&
                        environment !== 'production' &&
                        target === 'browser') {
                        plugins.push(new webpack.HotModuleReplacementPlugin());
                    }
                    // minify scripts in production in the browser
                    if (environment === 'production' && target === 'browser') {
                        plugins.push(new webpack.LoaderOptionsPlugin({
                            minimize: true,
                            debug: false
                        }));
                        plugins.push(new webpack.optimize.UglifyJsPlugin({
                            beautify: false,
                            mangle: {
                                screw_ie8: true,
                                keep_fnames: true
                            },
                            compress: {
                                warnings: false,
                                screw_ie8: true,
                                conditionals: true,
                                unused: true,
                                comparisons: true,
                                sequences: true,
                                dead_code: true,
                                evaluate: true,
                                if_return: true,
                                join_vars: true,
                                negate_iife: false
                            },
                            sourceMap: false
                        }));
                    }
                    config.devtool = environment === 'development' ? 'cheap-source-map' : false;
                    return [2 /*return*/, config];
            }
        });
    });
}
exports["default"] = Webpack;
function resolves() {
    var paths = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        paths[_i] = arguments[_i];
    }
    for (var i = 0; i < paths.length; i++) {
        var path = paths[i];
        try {
            return require.resolve(path);
        }
        catch (_e) { }
    }
    return null;
}
