"use strict";
exports.__esModule = true;
var route_js_1 = require("../../lib/route.js");
function default_1(source, sourceMap) {
    this.cacheable();
    var route = getRoute(this);
    if (!route) {
        return this.callback(new Error("couldn't find route for page"));
    }
    // This is a bit confusing because we only want to update once, but
    // need to satisfy 3 update conditions: 1st page load, HMR update,
    // and Router.push(...). We only want to update once when we load
    // the page and when there's an HMR update, but not when we async
    // re-route.
    //
    //   -> only do HMR in development mode
    //   if (module.hot) {
    //   -> accept the hot module, this is a noop when we're booting
    //     module.hot.accept()
    //
    //     -> if we're booting, this will be "idle", "apply" occurs
    //     -> when we've made a change, this will allow us to only
    //     -> target HMR updates
    //     if (module.hot.status() === 'apply') {
    //       const Router = require('elmo/router').default
    //       Router.update(Component, route).catch(console.error)
    //       return
    //     }
    //   }
    //
    //   -> This will not be triggered when we're using Router.push(...)
    //   -> there is no corresponding <script>.
    //   if (document.querySelector('script[src="${route}"]')) {
    //     const Router = require('elmo/router').default
    //     Router.update(Component, route).catch(console.error)
    //   }
    this.callback(null, "\n  " + source + "\n  ;(function (Component, route) {\n    Component = Component.default || Component\n\n    if (module.hot) {\n      module.hot.accept()\n\n      // this should only occur when we are using\n      // HMR, never when we boot or when we use Router.push(...)\n      if (module.hot.status() === 'apply') {\n        var Router = require('elmo').Router\n        Router.update(Component, route).catch(console.error)\n        return\n      }\n    }\n\n    // this should only get rendered on boot, never\n    // during HMR or when we use Router.push(...)\n    if (document.querySelector('script[src=\"" + route + "\"]')) {\n      var Router = require('elmo').Router\n      Router.update(Component, route)\n        .then(function () {\n          // this will signal to prerender.io compatible\n          // clients that the page has rendered successfully\n          window.prerenderReady = true\n        })\n        .catch(console.error)\n    }\n  })(require('" + this.resourcePath + "'), " + JSON.stringify(route) + ")\n  ", sourceMap);
}
exports["default"] = default_1;
function getRoute(loaderContext) {
    var root = loaderContext.options.context;
    var resourcePath = loaderContext.resourcePath;
    var dir = [root].find(function (d) { return resourcePath.indexOf(d) === 0; });
    if (!dir)
        return '';
    var route = route_js_1["default"](dir, resourcePath);
    if (route === '/')
        return '/index.js';
    return route + '/index.js';
}
