"use strict";
exports.__esModule = true;
var ts = require("typescript");
var transform_1 = require("../../styled-tsx/transform");
var tsconfig_1 = require("../config/tsconfig");
// import { compile } from 'pylo/server'
// const compiler: any = require('pylo/server').compile.default
// console.log(compiler.compile.default)
// compiler as any
var path_1 = require("path");
// const SRC_ROOT = join(__dirname, '..', '..')
function default_1(source, _sourceMap) {
    this.cacheable();
    var _a = this, resourcePath = _a.resourcePath, root = _a.options.context;
    var _b = this.query, environment = _b.environment, target = _b.target;
    // this is the project's tsconfig, not elmo/s
    var TSConfig = possRequire(path_1.join(root, 'tsconfig.json'));
    if (TSConfig instanceof Error) {
        var err_1 = new Error('invalid or missing tsconfig.json at: ' + path_1.join(root, 'tsconfig.json'));
        this.callback(err_1);
        return;
    }
    var errors = tsconfig_1.validate(TSConfig);
    if (errors.length) {
        var err_2 = new TypeError('\n' + errors.map(function (e) { return e.message; }).join('\n') + '\n');
        this.callback(err_2);
        return;
    }
    var hasRuntime = environment !== 'production' || target !== 'browser';
    // console.log('transpiling %s %s', root, resourcePath)
    var result = ts.transpileModule(source, {
        fileName: resourcePath,
        reportDiagnostics: true,
        transformers: {
            before: [
                visitor(transform_1["default"]({
                    runtime: hasRuntime
                }))
            ]
        },
        compilerOptions: TSConfig.compilerOptions
    });
    var err = error(result.diagnostics);
    if (err) {
        this.callback(err);
        return;
    }
    this.callback(null, result.outputText, result.sourceMapText);
}
exports["default"] = default_1;
// create an error if it exists
function error(diagnostics) {
    if (diagnostics === void 0) { diagnostics = []; }
    var errors = diagnostics.map(function (diagnostic) {
        if (!diagnostic.file || !diagnostic.start)
            return;
        var _a = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start), line = _a.line, character = _a.character;
        var reason = ts.flattenDiagnosticMessageText(diagnostic.messageText, '\n');
        return diagnostic.file.fileName + " (" + (line + 1) + "," + (character +
            1) + "): " + reason;
    });
    if (!errors.length)
        return;
    return new TypeError('\n' + errors.join('\n') + '\n');
}
function visitor(transform) {
    return function (context) {
        return function (sourceFile) {
            return visit(sourceFile);
            function visit(node) {
                var t = transform[node.kind];
                if (!t) {
                    return ts.visitEachChild(node, visit, context);
                }
                // enter a node
                if (typeof t === 'object' && t.enter) {
                    var tr = t.enter(node, context);
                    if (tr)
                        node = tr;
                }
                // if it's a function, treat it
                // the same way as t.enter
                if (typeof t === 'function') {
                    var tr = t(node, context);
                    if (tr)
                        node = tr;
                }
                // visit each child
                var exitNode = ts.visitEachChild(node, visit, context);
                // if we have an exit call function
                if (typeof t === 'object' && t.exit) {
                    var tr = t.exit(exitNode, context);
                    if (tr)
                        exitNode = tr;
                }
                return exitNode;
            }
        };
    };
}
function possRequire(req) {
    try {
        return require(req);
    }
    catch (e) {
        return e;
    }
}
