"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
var preact_1 = require("preact");
var Stringify = require("preact-render-to-string");
var Evaluate = require("eval");
var path_1 = require("path");
var server_1 = require("../../styled-tsx/server");
/**
 * Module Exports
 */
var Static = /** @class */ (function () {
    function Static(options, entrypaths) {
        this.options = options;
        this.entrypaths = entrypaths;
    }
    Static.prototype.build = function (entries) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            var documentPath, _a, elmoRoot, root, document, pages, pageData, entryPath;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        documentPath = this.entrypaths['document.js'];
                        _a = this.options, elmoRoot = _a.elmoRoot, root = _a.root;
                        document = evaluate(documentPath, entries['document.js'].source(), { require: require });
                        if (typeof document !== 'function') {
                            throw new Error('_document should be a Component. Please make sure you exported it');
                        }
                        pages = Object.keys(entries)
                            .filter(function (entry) { return /\.[tj]sx?/.test(path_1.extname(entry)); })
                            .filter(function (entry) { return /index\.[tj]sx?/.test(entry); });
                        return [4 /*yield*/, Promise.all(pages.map(function (filepath) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                var source, Component, props;
                                return tslib_1.__generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0:
                                            source = entries[filepath].source();
                                            Component = evaluate(path_1.resolve(root, filepath), source, { require: require });
                                            return [4 /*yield*/, getBuildProps(Component)];
                                        case 1:
                                            props = _a.sent();
                                            return [2 /*return*/, { filepath: filepath, props: props, source: source, Component: Component }];
                                    }
                                });
                            }); }))
                            // render HTML synchronously to avoid any race conditions
                            // with rewind
                        ];
                    case 1:
                        pageData = _b.sent();
                        // render HTML synchronously to avoid any race conditions
                        // with rewind
                        return [4 /*yield*/, Promise.all(pageData.map(function (_a) {
                                var filepath = _a.filepath, props = _a.props, Component = _a.Component;
                                return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                    var html;
                                    return tslib_1.__generator(this, function (_a) {
                                        switch (_a.label) {
                                            case 0: return [4 /*yield*/, renderHTML(document, {
                                                    elmoRoot: elmoRoot,
                                                    filepath: filepath,
                                                    Component: Component,
                                                    props: props
                                                })];
                                            case 1:
                                                html = _a.sent();
                                                addFile(entries, filepath, html);
                                                return [2 /*return*/];
                                        }
                                    });
                                });
                            }))
                            // we go ahead and delete all the JS files,
                            // because this is a server-side build and
                            // we only care about the client-side build
                            // results
                        ];
                    case 2:
                        // render HTML synchronously to avoid any race conditions
                        // with rewind
                        _b.sent();
                        // we go ahead and delete all the JS files,
                        // because this is a server-side build and
                        // we only care about the client-side build
                        // results
                        for (entryPath in entries) {
                            if (path_1.extname(entryPath) !== '.js')
                                continue;
                            delete entries[entryPath];
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    Static.prototype.apply = function (compiler) {
        var self = this;
        compiler.plugin('this-compilation', function (compiler) {
            compiler.plugin('optimize-assets', function (entries, done) {
                self
                    .build(entries)
                    .then(function () { return done(); })["catch"](function (e) {
                    done(e);
                });
            });
        });
    };
    return Static;
}());
exports["default"] = Static;
/**
 * Get the initial properties of the static build
 * // TODO: better inputs
 */
function getBuildProps(Component) {
    if (Component.getBuildProps) {
        return Component.getBuildProps({ build: true, path: '', query: {} });
    }
    else {
        return Promise.resolve({});
    }
}
/**
 * Render the HTML
 */
// TODO: not exactly sure how to type the Document when there's an evaluate()
function renderHTML(Document, params) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var Component, page, _a, head, style, docProps, vhtml, html;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    Component = params.Component;
                    page = Stringify(preact_1.h(Component, tslib_1.__assign({}, params.props)));
                    _a = rewindAll(params.elmoRoot), head = _a.head, style = _a.style;
                    return [4 /*yield*/, getBuildProps(Document)
                        // create a virtual DOM tree
                    ];
                case 1:
                    docProps = _b.sent();
                    vhtml = (preact_1.h(Document, { page: page, head: head, style: style, props: docProps || {}, filepath: params.filepath }));
                    html = '<!DOCTYPE html>' + Stringify(vhtml);
                    // ignore any <style scoped> or <Head /> in the document
                    rewindAll(params.elmoRoot);
                    return [2 /*return*/, html];
            }
        });
    });
}
/**
 * Add the entry file
 */
function addFile(entries, entry, html) {
    var name = path_1.basename(path_1.dirname(entry));
    entries[name + '/index.html'] = {
        source: function () { return html; },
        size: function () { return html.length; }
    };
    return null;
}
/**
 * Evaluate the component
 */
function evaluate(entry, source, globals) {
    var render;
    render = Evaluate(source, 
    /* filename: */ entry, 
    /* scope: */ globals, 
    /* includeGlobals: */ true);
    if (render.hasOwnProperty('default')) {
        render = render['default'];
    }
    return render;
}
// Okay so the reason for this is that when we call evaluate
// it may use a non-cached version of <Head /> or <Style/>
// depending on which module we require. These need to be
// pull from the cache to maintain their flush state.
// This function normalizes over all possible places
// you can get these modules from, so no matter where
// we require it we'll get the state.
function rewindAll(elmoRoot) {
    var dist = path_1.join(elmoRoot, 'dist', 'src');
    var src = path_1.join(elmoRoot, 'src');
    var cache = require.cache;
    var distHead = path_1.join(dist, 'lib', 'head.js');
    var srcHead = path_1.join(src, 'lib', 'head.ts');
    var distStyle = path_1.join(dist, 'styled-tsx', 'style.js');
    var srcStyle = path_1.join(src, 'styled-tsx', 'style.ts');
    var fn = null;
    var head = [];
    var style = [];
    for (var req in cache) {
        if (!cache[req])
            continue;
        switch (req) {
            case distHead:
                fn = cache[req].exports["default"];
                head = head.concat(fn.rewind());
                break;
            case srcHead:
                fn = cache[req].exports["default"];
                head = head.concat(fn.rewind());
                break;
            case distStyle:
                fn = cache[req].exports["default"];
                style = style.concat(server_1["default"](fn.flush()));
                break;
            case srcStyle:
                fn = cache[req].exports["default"];
                style = style.concat(server_1["default"](fn.flush()));
                break;
        }
    }
    return { head: head, style: style };
}
