"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
var path_1 = require("path");
var route_1 = require("../../lib/route");
var Regexp = require("path-to-regexp");
var globby = require("globby");
var fs_1 = require("mz/fs");
/**
 * Providers
 */
var redirects_1 = require("./redirects");
var package_1 = require("./package");
/**
 * Priority array
 */
var priority = ['_redirects', 'package.json'];
/**
 * Routing
 */
function Routing(root) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var pages, routes, pagemap, out, k, k;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, globby(path_1.join(root, 'pages', '**', '*.{js,jsx,ts,tsx}'))];
                case 1:
                    pages = _a.sent();
                    pages = pages.filter(function (page) { return path_1.basename(page)[0] !== '_'; });
                    return [4 /*yield*/, find(root)];
                case 2:
                    routes = _a.sent();
                    pagemap = pages.reduce(function (routes, p) {
                        var r = route_1["default"](root, p);
                        routes[r] = r;
                        return routes;
                    }, {});
                    out = [];
                    for (k in routes) {
                        out.push(compile(k, routes[k]));
                    }
                    for (k in pagemap) {
                        out.push(compile(k, pagemap[k]));
                    }
                    return [2 /*return*/, out];
            }
        });
    });
}
exports["default"] = Routing;
function find(root) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _this = this;
        var paths, existing, i, len;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    paths = priority.map(function (p) { return path_1.resolve(root, p); });
                    return [4 /*yield*/, Promise.all(paths.map(function (p) { return tslib_1.__awaiter(_this, void 0, void 0, function () { return tslib_1.__generator(this, function (_a) {
                            return [2 /*return*/, fs_1.exists(p)];
                        }); }); }))];
                case 1:
                    existing = _a.sent();
                    for (i = 0, len = existing.length; i < len; i++) {
                        if (!existing[i])
                            continue;
                        switch (priority[i]) {
                            case '_redirects':
                                return [2 /*return*/, redirects_1["default"](paths[i])];
                            case 'package.json':
                                return [2 /*return*/, package_1["default"](paths[i])];
                        }
                    }
                    return [2 /*return*/, {}];
            }
        });
    });
}
function compile(expr, path) {
    var params = [];
    var regexp = Regexp(expr, params);
    return {
        path: path,
        regexp: regexp.source,
        params: params.map(function (p) { return p.name; })
    };
}
