"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
var preact_1 = require("preact");
var render_1 = require("./render");
var components = [];
var Style = /** @class */ (function (_super) {
    tslib_1.__extends(Style, _super);
    function Style() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Style.prototype.componentWillMount = function () {
        components.push(this);
        update();
    };
    // To avoid FOUC, we process new changes
    // on `componentWillUpdate` rather than `componentDidUpdate`.
    Style.prototype.componentWillUpdate = function (nextProps) {
        update({
            instance: this,
            id: nextProps.id,
            css: nextProps.css
        });
    };
    Style.prototype.componentWillUnmount = function () {
        var i = components.indexOf(this);
        if (i < 0) {
            return;
        }
        components.splice(i, 1);
        update();
    };
    Style.prototype.render = function () {
        return null;
    };
    Style.flush = function () {
        var ret = stylesMap();
        components = [];
        return ret;
    };
    return Style;
}(preact_1.Component));
exports["default"] = Style;
// module.exports = Style
function update(updated) {
    render_1["default"](stylesMap(updated));
    // console.log('rendering', stylesMap(updated))
    // console.log(stylesMap(updated))
}
function stylesMap(updated) {
    var ret = {};
    for (var _i = 0, components_1 = components; _i < components_1.length; _i++) {
        var c = components_1[_i];
        if (updated && c === updated.instance) {
            // On `componentWillUpdate`
            // we use `styleId` and `css` from updated component rather than reading `props`
            // from the component since they haven't been updated yet.
            ret[updated.id] = updated.css;
        }
        else {
            ret[c.props.id] = c.props.css;
        }
    }
    return ret;
}
