"use strict";
exports.__esModule = true;
var ts = require("typescript");
var Stylis = require("stylis");
// setup stylis
var stylis = new Stylis();
stylis.set({
    cascade: false,
    compress: true
});
function Transform(opts) {
    var options = opts || {};
    options.runtime =
        typeof options.runtime === 'undefined' ? false : options.runtime;
    var importFrom = options.importFrom || 'styled-tsx';
    options.importName = options.importName || 'StyledTSX';
    var importName = options.importName;
    return _a = {},
        _a[ts.SyntaxKind.JsxElement] = {
            exit: function (node) {
                var el = node;
                var updates = [];
                var updated = [];
                var hashes = [];
                var children = el.children;
                for (var i = 0, len = children.length; i < len; i++) {
                    if (!isStyledJSX(options, children[i])) {
                        continue;
                    }
                    // we know at this point its an element
                    var child = children[i];
                    // ignore if no children
                    var css = getCSS(child);
                    if (!css)
                        continue;
                    var id = 'tsx-' + hash(css);
                    var compiled = stylis('.' + id, css);
                    // store the updates until we
                    // make changes to the parent
                    // first
                    updates.push([i, id, compiled]);
                    hashes.push(id);
                }
                // if this parent doesn't have a <style> tag, return
                if (!updates.length)
                    return;
                // update the class on the parent recursively
                var parent = updateClassRecursively(options, el, hashes.join(' '));
                // a parent can't really be self-closing
                // TODO: we should probably clean this up with a public and private
                // updateClassRecursively.
                if (!parent || parent.kind === ts.SyntaxKind.JsxSelfClosingElement)
                    return;
                // update children after we updated all the nodes
                children = parent.children;
                // now that we've added classes, we're going to
                // swap the style tag out. this order maintains
                // the references.
                var update = updates.shift();
                for (var i = 0, len = children.length; i < len; i++) {
                    if (!update || update[0] !== i) {
                        updated.push(children[i]);
                        continue;
                    }
                    else if (!options.runtime) {
                        continue;
                    }
                    var child = children[i];
                    // update the child
                    var styledTSX = ts.updateJsxElement(child, ts.updateJsxOpeningElement(child.openingElement, ts.createIdentifier(importName), ts.createJsxAttributes([
                        ts.createJsxAttribute(ts.createIdentifier('id'), ts.createLiteral(update[1])),
                        ts.createJsxAttribute(ts.createIdentifier('css'), ts.createLiteral(update[2]))
                    ])), [], ts.updateJsxClosingElement(child.closingElement, ts.createIdentifier(importName)));
                    updated.push(styledTSX);
                    update = updates.shift();
                }
                // update the jsx element with the new children
                var replacement = ts.updateJsxElement(el, parent.openingElement, updated, parent.closingElement);
                return replacement;
            }
        },
        _a[ts.SyntaxKind.SourceFile] = function (node, context) {
            if (!context)
                return;
            var opts = context.getCompilerOptions();
            var module = opts.module;
            // const StyledTSX = require('styled-tsx')
            if (module === ts.ModuleKind.CommonJS) {
                var file = node;
                var update = ts.updateSourceFileNode(file, [
                    ts.createVariableStatement(
                    /*modifiers*/ undefined, ts.createVariableDeclarationList([
                        ts.createVariableDeclaration(importName, 
                        /*type*/ undefined, ts.createPropertyAccess(ts.createCall(ts.createIdentifier('require'), [], [ts.createLiteral(importFrom)]), ts.createIdentifier('default')))
                    ]))
                ].concat(file.statements));
                return update;
            }
            // import StyledTSX from 'styled-tsx'
            if (module === ts.ModuleKind.ES2015 || module === ts.ModuleKind.ESNext) {
                var file = node;
                var update = ts.updateSourceFileNode(file, [
                    ts.createImportDeclaration(
                    /* decorators */ undefined, 
                    /* modifiers */ undefined, ts.createImportClause(ts.createIdentifier(importName), undefined), ts.createLiteral(importFrom))
                ].concat(file.statements));
                return update;
            }
            return node;
        },
        _a;
    var _a;
}
exports["default"] = Transform;
// function isScoped(
//   node: ts.JsxChild
// ): boolean {
//   // ignore non-jsx elements
//   if (node.kind !== ts.SyntaxKind.JsxElement) {
//     return false
//   }
//   const n = node.openingElement
//   // loop over properties
//   const props = n.attributes.properties || []
//   return true
// }
function isStyledJSX(options, node) {
    // ignore non-jsx elements
    if (node.kind !== ts.SyntaxKind.JsxElement) {
        return false;
    }
    var n = node.openingElement;
    if (n.tagName.kind !== ts.SyntaxKind.Identifier) {
        throw new Error('Expected an identifier but instead got a node of kind: ' + n.tagName.kind);
    }
    var tag = n.tagName;
    // if we've already renamed this tag, return a true
    // TODO: this is a hack
    if (tag.text === options.importName) {
        return { global: false, scoped: true };
    }
    // ignore any tags that aren't <style>
    if (tag.text !== 'style')
        return false;
    // loop over properties
    var props = n.attributes.properties || [];
    var isGlobal = false;
    var isScoped = false;
    for (var i = 0, len = props.length; i < len; i++) {
        var name_1 = props[i].name;
        if (!name_1)
            continue;
        var key = getText(name_1);
        isScoped = key === 'jsx' || key === 'scoped';
        isGlobal = key === 'global';
    }
    // ignore if not: <style jsx>, <style global>, <style scoped>
    if (!isScoped && !isGlobal)
        return false;
    return {
        global: isGlobal,
        scoped: isScoped
    };
}
function isHeadTag(node) {
    // jsx elements tags
    if (node.kind === ts.SyntaxKind.JsxElement) {
        var n = node.openingElement;
        var nodeName = n.tagName.getText();
        if (nodeName !== 'Head')
            return false;
        return true;
    }
    // self-closing tags
    if (node.kind === ts.SyntaxKind.JsxSelfClosingElement) {
        var nodeName = node.tagName.getText();
        if (nodeName !== 'Head')
            return false;
        return true;
    }
    return false;
}
function updateClassRecursively(options, node, hash) {
    // handle self-closing tags
    if (node.kind === ts.SyntaxKind.JsxSelfClosingElement) {
        return updateSelfClosingClass(node, hash);
    }
    var updated = [];
    var children = node.children;
    // traverse down the children first
    for (var i = 0, len = children.length; i < len; i++) {
        var ch = children[i];
        if (ch.kind !== ts.SyntaxKind.JsxElement &&
            ch.kind !== ts.SyntaxKind.JsxSelfClosingElement) {
            updated.push(ch);
            continue;
        }
        else if (isStyledJSX(options, ch)) {
            // ignore <style scoped> tags
            updated.push(ch);
            continue;
        }
        else if (isHeadTag(ch)) {
            // ignore <Head /> tags
            updated.push(ch);
            continue;
        }
        var newChild = updateClassRecursively(options, ch, hash);
        updated.push(newChild || ch);
    }
    var opening = updateClass(node.openingElement, hash);
    if (!opening)
        return;
    return ts.updateJsxElement(node, opening, updated, node.closingElement);
}
exports.updateClassRecursively = updateClassRecursively;
function updateSelfClosingClass(node, hash) {
    var attrs = node.attributes;
    var props = attrs.properties || [];
    for (var i = 0, len = props.length; i < len; i++) {
        // ignore spread attributes {...props}
        if (props[i].kind === ts.SyntaxKind.JsxSpreadAttribute)
            continue;
        var prop = props[i];
        // ignore expressions
        if (!prop.initializer)
            continue;
        // if (prop.initializer.kind === ts.SyntaxKind.JsxExpression) continue
        var name_2 = prop.name;
        if (!name_2)
            continue;
        var key = getText(name_2);
        if (key === 'class' || key === 'className') {
            var newAttr = null;
            switch (prop.initializer.kind) {
                case ts.SyntaxKind.StringLiteral:
                    newAttr = updateClassString(prop, hash);
                    break;
                case ts.SyntaxKind.JsxExpression:
                    newAttr = updateClassExpression(prop, hash);
                    break;
            }
            if (!newAttr)
                continue;
            // if (prop.initializer.kind === ts.SyntaxKind.StringLiteral) {
            //   newAttr = updateStringClass(prop, hash)
            //   // const classes = prop.initializer.text.split(/\s+/)
            //   // const newClass = ts.createLiteral(classes.concat(hash).join(' '))
            //   // const newAttr = ts.updateJsxAttribute(prop, name, newClass)
            // } else if ()
            // splice the prop
            var p = props
                .slice(0, i)
                .concat(newAttr)
                .concat(props.slice(i + 1));
            // update the attributes & exit early
            return ts.updateJsxSelfClosingElement(node, node.tagName, ts.updateJsxAttributes(node.attributes, p));
        }
    }
    var newProps = props.concat(ts.createJsxAttribute(ts.createIdentifier('className'), ts.createLiteral(hash)));
    return ts.updateJsxSelfClosingElement(node, node.tagName, ts.updateJsxAttributes(node.attributes, newProps));
}
function updateClass(node, hash) {
    var attrs = node.attributes;
    var props = attrs.properties || [];
    for (var i = 0, len = props.length; i < len; i++) {
        // ignore spread attributes {...props}
        if (props[i].kind === ts.SyntaxKind.JsxSpreadAttribute)
            continue;
        var prop = props[i];
        // ignore expressions
        if (!prop.initializer)
            continue;
        // if (prop.initializer.kind === ts.SyntaxKind.JsxExpression) continue
        var name_3 = prop.name;
        if (!name_3)
            continue;
        var key = getText(name_3);
        if (key === 'class' || key === 'className') {
            var newAttr = null;
            switch (prop.initializer.kind) {
                case ts.SyntaxKind.StringLiteral:
                    newAttr = updateClassString(prop, hash);
                    break;
                case ts.SyntaxKind.JsxExpression:
                    newAttr = updateClassExpression(prop, hash);
                    break;
            }
            if (!newAttr)
                continue;
            // if (prop.initializer.kind === ts.SyntaxKind.StringLiteral) {
            //   newAttr = updateStringClass(prop, hash)
            //   // const classes = prop.initializer.text.split(/\s+/)
            //   // const newClass = ts.createLiteral(classes.concat(hash).join(' '))
            //   // const newAttr = ts.updateJsxAttribute(prop, name, newClass)
            // } else if ()
            // splice the prop
            var p = props
                .slice(0, i)
                .concat(newAttr)
                .concat(props.slice(i + 1));
            // update the attributes & exit early
            return ts.updateJsxOpeningElement(node, node.tagName, ts.updateJsxAttributes(node.attributes, p));
        }
    }
    var newProps = props.concat(ts.createJsxAttribute(ts.createIdentifier('className'), ts.createLiteral(hash)));
    return ts.updateJsxOpeningElement(node, node.tagName, ts.updateJsxAttributes(node.attributes, newProps));
}
exports.updateClass = updateClass;
function updateClassString(node, hash) {
    var initializer = node.initializer;
    var existing = initializer.text.split(/\s+/);
    for (var i = 0, len = existing.length; i < len; i++) {
        if (existing[i].slice(0, 4) === 'tsx-')
            return node;
    }
    var text = existing.concat(hash).join(' ');
    var newClass = ts.createLiteral(text);
    return ts.updateJsxAttribute(node, node.name, newClass);
}
function updateClassExpression(node, hash) {
    var initializer = node.initializer;
    if (!initializer.expression) {
        return ts.createJsxAttribute(node.name, ts.createJsxExpression(undefined, ts.createLiteral(hash)));
    }
    var expr = initializer.expression;
    switch (expr.kind) {
        // {"hi"} {""} {''}
        case ts.SyntaxKind.StringLiteral:
            return ts.createJsxAttribute(node.name, ts.createJsxExpression(undefined, updateStringLiteral(expr, hash) || expr));
        // {`hi ${world}`}
        case ts.SyntaxKind.TemplateExpression:
            return ts.createJsxAttribute(node.name, ts.createJsxExpression(undefined, updateTemplateExpression(expr, hash) || expr));
        // {`hi`}
        case ts.SyntaxKind.NoSubstitutionTemplateLiteral:
            return ts.createJsxAttribute(node.name, ts.createJsxExpression(undefined, updateNoSubstitutionExpression(expr, hash) || expr));
        default:
            return ts.createJsxAttribute(node.name, ts.createJsxExpression(undefined, wrapExpression(expr, hash) || expr));
    }
}
function updateStringLiteral(node, hash) {
    var text = (node.text + ' ' + hash).trim();
    return ts.createLiteral(text);
}
function updateNoSubstitutionExpression(node, hash) {
    node.text = (node.text + ' ' + hash).trim();
    return node;
}
function updateTemplateExpression(node, hash) {
    if (!node.templateSpans.length)
        return;
    var lastSpan = node.templateSpans[node.templateSpans.length - 1];
    if (lastSpan.literal.kind !== ts.SyntaxKind.TemplateTail)
        return;
    // TODO: can't figure out how to update the ts.TemplateTail
    // in a non-destructive way
    var tail = lastSpan.literal;
    tail.text = tail.text + ' ' + hash;
    return node;
}
function wrapExpression(node, hash) {
    return ts.createBinary(ts.createParen(node), ts.SyntaxKind.PlusToken, ts.createLiteral(' ' + hash));
}
function getCSS(el) {
    if (!el.children.length)
        return '';
    var css = el.children[0];
    switch (css.kind) {
        // just a text element (probably single line)
        case ts.SyntaxKind.JsxText:
            return css.getText().trim();
        case ts.SyntaxKind.JsxExpression:
            var expr = css.expression;
            if (!expr)
                return '';
            if (expr.kind !== ts.SyntaxKind.NoSubstitutionTemplateLiteral) {
                throw new Error('<style scoped> does not support templates ${...}');
            }
            var literal = expr;
            return literal.text || '';
    }
    return '';
}
function getText(node) {
    switch (node.kind) {
        case ts.SyntaxKind.NoSubstitutionTemplateLiteral:
            return node.text;
        case ts.SyntaxKind.Identifier:
            return node.text;
        case ts.SyntaxKind.StringLiteral:
            return node.text;
        case ts.SyntaxKind.NumericLiteral:
            return node.text;
        default:
            return '';
    }
}
/**
 * JS Implementation of MurmurHash2
 *
 * @author <a href="mailto:gary.court@gmail.com">Gary Court</a>
 * @see http://github.com/garycourt/murmurhash-js
 * @author <a href="mailto:aappleby@gmail.com">Austin Appleby</a>
 * @see http://sites.google.com/site/murmurhash/
 *
 * @param {string} str ASCII only
 * @return {string} Base 36 encoded hash result
 */
function hash(str) {
    var l = str.length;
    var h = l;
    var i = 0;
    var k;
    while (l >= 4) {
        k =
            (str.charCodeAt(i) & 0xff) |
                ((str.charCodeAt(++i) & 0xff) << 8) |
                ((str.charCodeAt(++i) & 0xff) << 16) |
                ((str.charCodeAt(++i) & 0xff) << 24);
        k = (k & 0xffff) * 0x5bd1e995 + ((((k >>> 16) * 0x5bd1e995) & 0xffff) << 16);
        k ^= k >>> 24;
        k = (k & 0xffff) * 0x5bd1e995 + ((((k >>> 16) * 0x5bd1e995) & 0xffff) << 16);
        h =
            ((h & 0xffff) * 0x5bd1e995 +
                ((((h >>> 16) * 0x5bd1e995) & 0xffff) << 16)) ^
                k;
        l -= 4;
        ++i;
    }
    switch (l) {
        case 3:
            h ^= (str.charCodeAt(i + 2) & 0xff) << 16;
            break;
        case 2:
            h ^= (str.charCodeAt(i + 1) & 0xff) << 8;
            break;
        case 1:
            h ^= str.charCodeAt(i) & 0xff;
            h =
                (h & 0xffff) * 0x5bd1e995 + ((((h >>> 16) * 0x5bd1e995) & 0xffff) << 16);
            break;
    }
    h ^= h >>> 13;
    h = (h & 0xffff) * 0x5bd1e995 + ((((h >>> 16) * 0x5bd1e995) & 0xffff) << 16);
    h ^= h >>> 15;
    return (h >>> 0).toString(36);
}
