// const has = Object.prototype.hasOwnProperty

/**
 * Decode a URI encoded string.
 *
 * @param {String} input The URI encoded string.
 * @returns {String} The decoded string.
 * @api private
 */
function decode(input: string): string {
  return decodeURIComponent(input.replace(/\+/g, ' '))
}

export type QS = { [s: string]: string | boolean }

/**
 * Simple query string parser.
 *
 * @param {String} query The query string that needs to be parsed.
 * @returns {Object}
 * @api public
 */
export function parse(query: string): QS {
  if (!query) return {}
  var parser = /([^=?&]+)=?([^&]*)/g
  var result: QS = {}
  var part

  // Little nifty parsing hack, leverage the fact that RegExp.exec increments
  // the lastIndex property so we can continue executing this loop until we've
  // parsed all results.
  while ((part = parser.exec(query))) {
    var val: string | boolean = decode(part[2])
    if (val === '') val = true
    if (val === 'false') val = false
    result[decode(part[1])] = val
  }

  return result
}

/**
 * Transform a query string to an object.
 *
 * @param {Object} obj Object that should be transformed.
 * @param {String} prefix Optional prefix.
 * @returns {String}
 * @api public
 */
// export function stringify(obj: Object, prefix: string) {
//   prefix = prefix || ''
//   var pairs = []

//   //
//   // Optionally prefix with a '?' if needed
//   //
//   if (typeof prefix !== 'string') prefix = '?'

//   for (var key in obj) {
//     if (has.call(obj, key)) {
//       pairs.push(encodeURIComponent(key) + '=' + encodeURIComponent(obj[key]))
//     }
//   }

//   return pairs.length ? prefix + pairs.join('&') : ''
// }
