import * as pascal from 'to-pascal-case'

export function createPage(pageName: string): string {
  const className = pascal(pageName)
  return `import { h, Component } from 'preact'
import { Head } from 'elmo'

export default class ${className} extends Component {
  static defaultProps = {
    greeting: 'hello'
  }

  static async getInitialProps () {
    return {
      greeting: 'hola'
    }
  }

  constructor(props) {
    super(props)
    this.state = {}
  }

  render(props, state) {
    return (
      <div class='page'>
        <Head>
          <title>hello ${pageName}!</title>
        </Head>
        {props.greeting} ${pageName}!
      </div>
    )
  }
}
`
}

export function createTSPage(pageName: string): string {
  const className = pascal(pageName)
  return `import { h, Component, ComponentProps } from 'preact'
import { Head } from 'elmo'

// ${pageName} props
interface Props extends ComponentProps {
  greeting: string
}

// ${pageName} state
interface State {
}

export default class ${className} extends Component<Props, State> {
  static defaultProps: Props = {
    greeting: 'hello'
  }

  static async getInitialProps(): Promise<Props> {
    return {
      greeting: 'hola'
    }
  }

  constructor (props: Props) {
    super(props)
    this.state = {}
  }

  render (props: Props, _state: State) {
    return (
      <div class='page'>
        <Head>
          <title>{props.greeting} ${pageName}!</title>
        </Head>
        {props.greeting} ${pageName}!
        <style scoped>{\`
          .page {
            height: 100vh;
            width: 100vw;
            background: #fefefe;
          }
        \`}</style>
      </div>
    )
  }
} 
`
}