export function validate(config: any): Array<Error> {
  const errors = []
  if (!config.compilerOptions) {
    errors.push(new Error(`Your tsconfig must have "compilerOptions"`))
    return errors
  }
  if (!/^(es6|es2015|esnext)$/i.test(config.compilerOptions.module || '')) {
    errors.push(
      new Error(
        `Your tsconfig's "compilerOptions" must have a "module" parameter of either "es6", "es2015", or "esnext" to work with elmo`
      )
    )
  }
  if (config.compilerOptions.moduleResolution !== 'node') {
    errors.push(
      new Error(
        `Your tsconfig's "compilerOptions" must contain { "moduleResolution": "node" } to work with elmo`
      )
    )
  }

  if (
    !config.compilerOptions.lib ||
    !~config.compilerOptions.lib.indexOf('dom') ||
    (!~config.compilerOptions.lib.indexOf('es2015') &&
      !~config.compilerOptions.lib.indexOf('es6'))
  ) {
    errors.push(
      new Error(
        `Your tsconfig's "compilerOptions" must contain a minimum "lib" of { "lib": ["dom", "es6"] } to work with elmo`
      )
    )
  }

  if (!config.compilerOptions.jsx || config.compilerOptions.jsx !== 'react') {
    errors.push(
      new Error(
        `Your tsconfig's "compilerOptions" must contain { "jsx": "react" }`
      )
    )
  }

  if (
    !config.compilerOptions.jsxFactory ||
    config.compilerOptions.jsxFactory !== 'h'
  ) {
    errors.push(
      new Error(
        `Your tsconfig's "compilerOptions" must contain { "jsxFactory": "h" }`
      )
    )
  }

  // this is temporary until preact includes a better typescript definition
  // see: https://github.com/developit/preact/pull/737
  if (
    !config.compilerOptions.baseUrl ||
    !config.compilerOptions.paths ||
    !~config.compilerOptions.paths['*'] ||
    !~config.compilerOptions.paths['*'].indexOf('node_modules/elmo/types/*')
  ) {
    errors.push(
      new Error(
        `Your tsconfig's "compilerOptions" must contain { "base": ".", "paths": { "*": ["node_modules/elmo/types/*"] } }`
      )
    )
  }

  return errors
}
