type elementMap = { [id: string]: Element }
type map = { [id: string]: string }
type diff = [map, map]
let prev: { [id: string]: string } = {}
const tags: elementMap = {}

export default (typeof window === 'undefined' ? renderOnServer : renderOnClient)

export function renderOnServer() {}

export function renderOnClient(styles: { [id: string]: string }) {
  patch(diff(prev, styles))
  prev = styles
}

function diff(a: map, b: map): diff {
  // added
  const added: map = {}
  for (let id in b) {
    if (!b.hasOwnProperty(id)) continue
    if (!a[id]) added[id] = b[id]
  }

  // removed
  const removed: map = {}
  for (let id in a) {
    if (!a.hasOwnProperty(id)) continue
    if (!b[id]) removed[id] = a[id]
  }

  return [added, removed]
}

const fromServer: { [id: string]: Element | null } = {}

function patch([added, removed]: diff) {
  // add the new tags
  for (let id in added) {
    // Avoid duplicates from server-rendered markup
    if (!fromServer[id]) {
      fromServer[id] = document.getElementById(id)
    }
    tags[id] = fromServer[id] || makeStyleTag(added[id])
  }

  // remove the old tags
  for (let id in removed) {
    const t = tags[id]
    delete tags[id]

    t.parentNode && t.parentNode.removeChild(t)
    // Avoid checking the DOM later on
    delete fromServer[id]
  }
}

function makeStyleTag(css: string): Element {
  const tag = document.createElement('style')
  tag.appendChild(document.createTextNode(css))
  const head = document.head || document.getElementsByTagName('head')[0]
  const firstStyle = document.getElementsByTagName('style')[0]
  head.insertBefore(tag, firstStyle)
  return tag
}
